import os.path as path
import scipy.io.wavfile as wav
import scipy.signal as sig
import numpy as np
import numpy.matlib as matlib
import matplotlib.pyplot as plt

#Provided functions
def getWindow(frame_length,windowing_type):
    if (windowing_type == 'rect'):
        window = np.ones(frame_length)
    elif (windowing_type == 'hann'):
        window = np.hanning(frame_length)
    elif (windowing_type == 'cosine'):
        window = np.sqrt(np.hanning(frame_length))
    elif (windowing_type == 'hamming'):
        window = np.hamming(frame_length)
    else:
        print('Windowing function not supported')
        exit()

    return window.reshape(-1,1)

def WGN(data_len, snr):
    #Add white gaussian noise to the signal with the defined snr.

    noise = np.random.normal(0,1,data_len)
    pow_ratio = np.power(10,(snr/20))
    noise_red = noise*pow_ratio

    return noise_red

def read_targets():
    #Target files must be in the same directory
    with open('output_targets','r') as f:
        data = f.read()
    
    targets = np.array([int(i) for i in data.split()])
    targets = targets.reshape((1,len(targets)))
    return targets

#Complete these functions

def noiseEst(data_matrix,est_type):
    # If estimation type is ideal noise, return back the true noise value. Else, if estimation type is avg_noise_model, get the average noise model by computing the mean of the noise type. The dimesions of 'ret' should be same as the dimensions of the input data_matrix
    if (est_type == 'avg_noise_model'):
        noise_mat = None
        noise_freq = None
        #Get the average-noise model
        avg_noise = None
        # Return model of the same dimensions as input noise matrix
        ret = None
    elif (est_type == 'ideal_noise'):
        #Get the ideal noise
        noise_mat = None
        noise_freq = None
        ret = None
    else:
        print('Wrong noise type')
        exit()
    
    return ret


def spectralSub(frame_matrix, hop_size, window_type, noise_est, original_signal_length):
    """
    Return the enhacned signal after spectral subtraction
    """

    #Setting initial variables
    frame_length = len(frame_matrix)
    fftlen = int(np.around(frame_length/2)+1)
    hwin = getWindow(frame_length,window_type)
    xest = np.zeros((original_signal_length,1))

    #Performging enhancement for each frame

    for winix in range(len(frame_matrix[0])):
        start = int(winix*hop_size)
        stop = int(np.minimum(start+frame_length,original_signal_length))
        
        #Applying FFT and leaving out the symmetrical second half (or use numpy rfft)
        Y_freq = None
        #Applyin filtering - Spectral Subtraction
        SpecSub_filt = None
        X_freq_est = None
        #Reconstruction and inverse-FFT
        xwinest = None
        #Overlap-add to get back the entire signal
        xest[start:stop] = None

    return xest

def wiener(frame_matrix, hop_size, window_type, noise_est, original_signal_length):
    """
    Return the enhacned signal after Wiener filtering
    """

    #Setting initial variables
    frame_length = len(frame_matrix)
    fftlen = int(np.around(frame_length/2)+1)
    hwin = getWindow(frame_length,window_type)
    xest = np.zeros((original_signal_length,1))

    #Performging enhancement for each frame

    for winix in range(len(frame_matrix[0])):
        start = int(winix*hop_size)
        stop = int(np.minimum(start+frame_length,original_signal_length))
        #Applying FFT and leaving out the symmetrical second half (or use numpy rfft)
        Y_freq = None
        #Applyin filtering - Spectral Subtraction
        Wiener_filt = None
        X_freq_est = None
        #Reconstruction and inverse-FFT
        xwinest = None
        #Overlap-add to get back the entire signal
        xest[start:stop] = None
    return xest

def linear(frame_matrix, hop_size, window_type, noise_est, original_signal_length):
    """
    Return the enhacned signal after linear filtering
    """

    #Setting initial variables
    frame_length = len(frame_matrix)
    fftlen = int(np.around(frame_length/2)+1)
    hwin = getWindow(frame_length,window_type)
    xest = np.zeros((original_signal_length,1))

    #Performging enhancement for each frame

    for winix in range(len(frame_matrix[0])):
        start = int(winix*hop_size)
        stop = int(np.minimum(start+frame_length,original_signal_length))
        #Applying FFT and leaving out the symmetrical second half (or use numpy rfft)
        Y_freq = None
        #Applyin filtering - Spectral Subtraction
        Wiener_filt = None
        X_freq_est = None
        #Reconstruction and inverse-FFT
        xwinest = None
        #Overlap-add to get back the entire signal
        xest[start:stop] = None

    return xest


def vadEnhance(frame_matrix, hop_size, window_type, noise_est, original_signal_length, vad_target):
    """
    Return the enhacned signal after Wiener filtering with VAD trigger
    """

    #Setting initial variables
    frame_length = len(frame_matrix)
    fftlen = int(np.around(frame_length/2)+1)
    hwin = getWindow(frame_length,window_type)
    xest = np.zeros((original_signal_length,1))

    #Performging enhancement for each frame

    for winix in range(len(frame_matrix[0])):
        start = int(winix*hop_size)
        stop = int(np.minimum(start+frame_length,original_signal_length))
        #Applying FFT and leaving out the symmetrical second half (or use numpy rfft)
        Y_freq = None
        #Applyin filtering - Spectral Subtraction
        if vad_target(winix):
            VAD_filt = None
        else:
            VAD_filt = None
        
        X_freq_est = None
        #Reconstruction and inverse-FFT
        xwinest = None
        #Overlap-add to get back the entire signal
        xest[start:stop] = None

    return xest

def snrGlb(clean, enhanced):
    if not (len(clean) == len(enhanced)):
        print('Length of signals do not match')
        exit()
    
    #Compute the global SNR in dB
    ret = None

    return ret

def snrSeg(clean,enhanced):
    if not (len(clean) == len(enhanced)):
        print('Length of signals do not match')
        exit()
    
    #Compute the segmental-SNR in dB
    noise = None
    ret = None

    return ret